package org.gel.mauve.dcjx;

public interface Constants {

	//public static final String CIRCULAR_CHAR = "*";
	
	public static final String TELOMERE = "<>";
	
	public static final String TAIL_TAG = "_t";
	
	public static final String HEAD_TAG = "_h";
}
