package org.gel.mauve;

public class ViewerMode {
	public final static ViewerMode NORMAL = new ViewerMode ();

	public final static ViewerMode ZOOM = new ViewerMode ();

	private ViewerMode () {
	};
}
